// Generated by CoffeeScript 2.3.2
var S_IFDIR, S_IFLNK, S_IFMT, S_IFREG, chmodSync, copyFile, copyTree, errno, follow, fs, getFileType, lstatSync, mkdirSync, os, path, readFileSync, readdirSync, readlinkSync, removeTree, renameSync, resolve, rmdirSync, statSync, symlinkSync, uhoh, unlinkSync, utimesSync, writeFileSync;

({chmodSync, lstatSync, mkdirSync, readdirSync, readlinkSync, readFileSync, renameSync, rmdirSync, statSync, symlinkSync, unlinkSync, utimesSync, writeFileSync} = require('fs'));

({S_IFMT, S_IFREG, S_IFDIR, S_IFLNK} = require('fs').constants);

errno = require('./errno');

path = require('path');

os = require('os');

fs = exports;

fs.stat = function(name) {
  return statSync(resolve(name));
};

fs.lstat = function(name) {
  return lstatSync(resolve(name));
};

fs.read = function(name, enc) {
  var type;
  name = resolve(name);
  if (!(type = getFileType(name))) {
    uhoh(`Path does not exist: '${name}'`, 'NOT_REAL');
  }
  if (type === S_IFDIR) {
    uhoh(`Path is not readable: '${name}'`, 'NOT_FILE');
  }
  if (enc === void 0) {
    enc = "utf8";
  }
  return readFileSync(name, enc);
};

fs.readJson = function(name) {
  return JSON.parse(fs.read(name));
};

fs.list = function(name) {
  var type;
  name = resolve(name);
  if (!(type = getFileType(name))) {
    uhoh(`Path does not exist: '${name}'`, 'NOT_REAL');
  }
  if (type !== S_IFDIR) {
    uhoh(`Path is not a directory: '${name}'`, 'NOT_DIR');
  }
  return readdirSync(name);
};

fs.follow = function(name, recursive) {
  var type;
  name = resolve(name);
  if (!(type = getFileType(name))) {
    uhoh(`Path does not exist: '${name}'`, 'NOT_REAL');
  }
  if (type === S_IFLNK) {
    return follow(name, recursive);
  } else {
    return name;
  }
};

fs.exists = function(name) {
  return getFileType(resolve(name)) !== null;
};

fs.isFile = function(name) {
  return getFileType(resolve(name)) === S_IFREG;
};

fs.isDir = function(name) {
  return getFileType(resolve(name)) === S_IFDIR;
};

fs.isLink = function(name) {
  return getFileType(resolve(name)) === S_IFLNK;
};

fs.readPerms = function(name) {
  return '0' + (fs.stat(name).mode & parseInt('777', 8)).toString(8);
};

fs.touch = function(name) {
  var time;
  name = resolve(name);
  if (getFileType(name) === null) {
    return writeFileSync(name, '');
  }
  time = Date.now() / 1000;
  return utimesSync(name, time, time);
};

fs.chmod = function(name, mode) {
  var type;
  name = resolve(name);
  if (!(type = getFileType(name))) {
    uhoh(`Path does not exist: '${name}'`, 'NOT_REAL');
  }
  if (type === S_IFDIR) {
    uhoh(`Path is a directory: '${name}'`, 'NOT_FILE');
  }
  return chmodSync(name, mode);
};

fs.link = function(name, target) {
  name = resolve(name);
  if (getFileType(name) === null) {
    return symlinkSync(target, name);
  }
  return uhoh(`Path already exists: '${name}'`, 'PATH_EXISTS');
};

fs.write = function(name, content) {
  name = resolve(name);
  if (getFileType(name) !== S_IFDIR) {
    return writeFileSync(name, content);
  }
  return uhoh(`Path is a directory: '${name}'`, 'NOT_FILE');
};

fs.mkdir = function(name, mode) {
  var type;
  name = resolve(name);
  if (!(type = getFileType(name))) {
    fs.mkdir(path.dirname(name));
    return mkdirSync(name, {mode});
  }
  if (type === S_IFLNK) {
    type = getFileType(follow(name, true));
  }
  if (type !== S_IFDIR) {
    return uhoh(`Path already exists: '${name}'`, 'PATH_EXISTS');
  }
};

fs.rename = function(src, dest) {
  src = resolve(src);
  if (!getFileType(src)) {
    uhoh(`Path does not exist: '${src}'`, 'NOT_REAL');
  }
  dest = resolve(dest);
  if (getFileType(dest)) {
    uhoh(`Path already exists: '${dest}'`, 'PATH_EXISTS');
  }
  fs.mkdir(path.dirname(dest));
  return renameSync(src, dest);
};

fs.remove = function(name, recursive) {
  var type;
  name = resolve(name);
  if (type = getFileType(name)) {
    if (type === S_IFDIR) {
      if (recursive) {
        return removeTree(name);
      } else {
        return rmdirSync(name);
      }
    } else {
      return unlinkSync(name);
    }
  }
};

fs.copy = function(srcPath, destPath) {
  var destMode, type;
  srcPath = resolve(srcPath);
  destPath = resolve(destPath);
  if (!(type = getFileType(srcPath))) {
    uhoh(`Cannot \`copy\` non-existent path: '${srcPath}'`, 'NOT_REAL');
  }
  if (type === S_IFDIR) {
    return copyTree(srcPath, destPath);
  }
  destMode = getFileType(destPath);
  if (destMode === S_IFDIR) {
    destPath = path.join(destPath, path.basename(srcPath));
    destMode = getFileType(destPath);
  }
  if (destMode) {
    if (destMode === S_IFDIR) {
      uhoh(`Cannot overwrite directory path: '${destPath}'`, 'PATH_EXISTS');
    } else {
      unlinkSync(destPath);
    }
  }
  return copyFile(srcPath, destPath);
};


// Internal

uhoh = function(msg, why) {
  var err;
  err = Error(msg);
  err.code = errno[why] || 0;
  Error.captureStackTrace(err, uhoh);
  throw err;
};

resolve = function(name) {
  if (name[0] === '~') {
    return os.homedir() + name.slice(1);
  } else {
    return path.resolve(name);
  }
};

getFileType = function(name) {
  var e;
  try {
    return lstatSync(name).mode & S_IFMT;
  } catch (error) {
    e = error;
    return null;
  }
};

// Recursive symlink resolution
follow = function(link, recursive) {
  var name, prev, reads, type, validate;
  name = link;
  reads = 0;
  if (typeof recursive === 'function') {
    validate = recursive;
    recursive = true;
  }
  while (++reads) {
    prev = name;
    name = readlinkSync(prev);
    if (!path.isAbsolute(name)) {
      name = path.resolve(path.dirname(prev), name);
    }
    if (validate && !validate(name)) {
      return prev;
    }
    if (!recursive) {
      return name;
    }
    if (!(type = getFileType(name))) {
      uhoh(`Symlink leads nowhere: '${link}'`, 'NOT_REAL');
    }
    if (type !== S_IFLNK) {
      return name;
    }
    if (reads === 10) {
      uhoh(`Too many symlinks: '${link}'`, 'LINK_LIMIT');
    }
  }
};

copyFile = function(srcPath, destPath) {
  fs.mkdir(path.dirname(destPath));
  if (getFileType(srcPath) === S_IFLNK) {
    return symlinkSync(readlinkSync(srcPath), destPath);
  }
  writeFileSync(destPath, readFileSync(srcPath));
  return chmodSync(destPath, fs.readPerms(srcPath));
};

// Recursive tree copies.
copyTree = function(srcDir, destDir) {
  var destMode;
  destMode = getFileType(destDir);
  // Remove the file under our new path, if needed.
  if (destMode && destMode !== S_IFDIR) {
    unlinkSync(destDir);
  }
  // Create the directory, if needed.
  if (destMode !== S_IFDIR) {
    fs.mkdir(destDir);
  }
  return readdirSync(srcDir).forEach(function(file) {
    var destPath, srcPath;
    srcPath = path.join(srcDir, file);
    destPath = path.join(destDir, file);
    if (getFileType(srcPath) === S_IFDIR) {
      return copyTree(srcPath, destPath);
    }
    if (destMode = getFileType(destPath)) {
      if (destMode === S_IFDIR) {
        removeTree(destPath);
      } else {
        unlinkSync(destPath);
      }
    }
    return copyFile(srcPath, destPath);
  });
};

// Recursive tree deletion.
removeTree = function(name) {
  var child, i, len, ref;
  ref = readdirSync(name);
  for (i = 0, len = ref.length; i < len; i++) {
    child = ref[i];
    child = path.join(name, child);
    if (getFileType(child) === S_IFDIR) {
      removeTree(child);
    } else {
      unlinkSync(child);
    }
  }
  return rmdirSync(name);
};

(function() {  // Expose error codes.
  var def, des, why;
  def = Object.defineProperty;
  des = {
    value: 0
  };
  for (why in errno) {
    des.value = errno[why];
    def(fs, why, des);
  }
})();
